#!/usr/bin/python
#coding: utf-8

import gtk, sys
from pyapplets import Pyapplet
from elementtree.ElementTree import *

bonobostrings = {
'exe_loc' : 'mhdbrno_applet.py',
'id' : 'GNOME_MHDapplet',
'iconpath' : 'mhdbrno.png',
'exe_name' : 'MHD Brno',
'exe_desc' : 'MHD Brno',
'fact_name' : 'MHD Brno',
'fact_desc' : 'Tento applet ukazuje čas zbývající do odjezdu šaliny'}

import urllib2, re, time, bisect, sys, os, gtk, gobject
from HTMLParser import HTMLParser

class IDOS_MHD(HTMLParser):

    def __init__(self, name):
        html = self.get_html(name)
        sone = 'Sobota + Ne' in html
        HTMLParser.__init__(self)
        self.odjezdy = [[], [], []]
#        self.odjezdy = [[]] * 3 #WARNING! NEVER USE!
        if sone:
            self.tdnum_to_day = {61:0, 65:1}
        else:
            self.tdnum_to_day = {57:0, 69:1, 73:2}
        self.thisline = []
        self.tdcounter = 0
        self.within_td = False
        self.regexp_num = re.compile(r'[^ \d]')
        self.feed(html)
        self.close()
        if sone: self.odjezdy[2] = self.odjezdy[1]
        self.__make_odj_sec()

    def get_html(self, filepath):
        html = file(filepath).read()
        html = html.replace('</>', '</b>') #IDOS HACK-they have error
                                           #in the html source
        return html

    def __make_odj_sec(self):
        def odj_to_sec(odj):
            result = []
            for hour, mins in enumerate(odj):
                secs = [hour * 3600 + min * 60 for min in mins]
                result.extend(secs)
            return result
        self.odj_sec = []
        for odjezd in self.odjezdy:
            self.odj_sec.append(odj_to_sec(odjezd))

    def handle_starttag(self, tag, attrs):
        if tag == 'td':
            self.tdcounter = self.tdcounter + 1
            self.within_td = True
            try:
                day = self.tdnum_to_day[self.tdcounter]
                self.thisodj = self.odjezdy[day]
            except KeyError:
                self.within_td = False
        if self.within_td:
            if tag == 'br':
                self.thisodj.append(self.thisline)
                self.thisline = []

    def handle_data(self, data):
        if self.within_td:
            data = re.sub(self.regexp_num, '', data).split()
            self.thisline.extend([int(x) for x in data])

    def handle_endtag(self, tag):
        if tag == 'td':
            self.within_td = False

    def print_schedule(self):
        def __ints_to_str(ints):
            return ' '.join(['%02d' %x for x in ints])
        for hour in range(24):
            prac = __ints_to_str(self.odjezdy[0][hour])
            sob = __ints_to_str(self.odjezdy[1][hour])
            ned = __ints_to_str(self.odjezdy[2][hour])
            all = '\t'.join([prac, sob, ned])
            all = all.expandtabs(30)
            print '%02d: %s' %(hour, all)

    def find_dep_wday(self, wday, dsec):
        i = {0:0, 1:0, 2:0, 3:0, 4:0, 5:1, 6:2}[wday]
        departures_today = self.odj_sec[i]
        dep_index = bisect.bisect(departures_today, dsec)
        try:
            dep_sec = departures_today[dep_index]
            return dep_sec - dsec
        except IndexError:
            print u'Dnes už nic nejede'
            sec = self.find_dep_wday((wday + 1) % 7, dsec - 86400)
            return sec

    def find_departure(self, epochsec = None):
        '''returns absolute time of next departure'''
        if epochsec == None:
            epochsec = time.time()
        now = time.localtime(epochsec)
        now_h = now.tm_hour
        now_m = now.tm_min
        now_s = epochsec % 60
        now_wday = now.tm_wday
        dsec = 3600 * now_h + 60 * now_m + now_s
        return self.find_dep_wday(now_wday, dsec) + epochsec



class Applet(Pyapplet):
    def __init__(self):
        Pyapplet.__init__(self, bonobostrings)
        self.parsecfg()
        vbox = gtk.VBox(False, 0)
        self.ways = []
        for note, htmlfile, delay in self.transports:
            mhd = IDOS_MHD(htmlfile)
            label = gtk.Label()
            label.show()
            vbox.pack_start(label, True, True, 0)
            self.ways.append((note, delay, label, mhd))
        self.update()
        vbox.show()
        self.widget = vbox


    def parsecfg(self):
        root = ElementTree(file = self.get_my_xml_cfg()).getroot()
        self.transports = []
        cfgdir = self.get_cfgdir()

        for elem in root.getchildren():
            transport = elem.attrib
            cesta = float(transport['cesta']) * 60
            htmlfile = os.path.join(cfgdir, transport['htmlfile'])
            label = transport['label']
            self.transports.append((label, htmlfile, cesta))


    def update(self, widget = None):
        epochsec = time.time()
        for note, delay, label, mhd in self.ways:
            dep = mhd.find_departure(epochsec + delay)
            min_to_leave = int(dep - epochsec - delay) / 60
            text = '%s%d' %(note, min_to_leave)
            label.set_text(text)
        #milliseconds till whole minute
        ms = 2 + int(1000 * (60 - epochsec % 60))
        #print 'waiting %d milliseconds' %ms
        self.timeout = gobject.timeout_add(ms, self.update, None)


if __name__ == '__main__':
    test = (len(sys.argv) == 1)
    appl = Applet()
    #appl.save_in_file('GNOME_MHDapplet.server')
    appl.run(test)
